package com.nantian.cloud.dbcapp.mapper.record;

import com.nantian.cloud.dbcapp.dto.ElectricityDataDto;
import com.nantian.cloud.dbcapp.vo.CoalVo;
import com.nantian.cloud.dbcapp.vo.ElectricityDataVo;
import com.nantian.cloud.dbcapp.vo.KeyValueVO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 *
 *
 * @author
 * @date 2018-09-27 15:20:44
 */
@Mapper
public interface ElectricityMapper {

    Integer getTotal(@Param("area") String area, @Param("inde1") String inde1);

    Integer getInfo(@Param("area") String area, @Param("industry") String industry, @Param("time") String time);


    List<ElectricityDataVo> selectElectricityData2019(@Param("area")String area, @Param("industry")String industry, @Param("preTime")String preTime, @Param("endTime")String endTime);

    List<ElectricityDataVo> selectElectricityData2020(@Param("area")String area, @Param("industry")String industry, @Param("preTime")String s, @Param("endTime")String endTime);

    List<ElectricityDataVo> selectEnterpriseData2019(@Param("enterpriseCode") String enterpriseCode, @Param("preTime") String preTime, @Param("endTime") String endTime);

    List<ElectricityDataVo> selectEnterpriseData2020(@Param("enterpriseCode") String enterpriseCode, @Param("preTime") String preTime, @Param("endTime") String endTime);

    Integer countCmpNum2019(@Param("area")String areaName, @Param("industry")String industryName);

    Integer countCmpNum2020(@Param("area")String areaName, @Param("industry")String industryName);

    List<ElectricityDataDto> selectByDay(String enterpriseCode, String time);

    List<ElectricityDataDto> selectCollectEleByDay(@Param("enterpriseCode") String enterpriseCode,@Param("time") String time, @Param("dataCode") String dataCode);

    List<ElectricityDataVo> selectCollectEleByMonth(@Param("enterpriseCode") String enterpriseCode, @Param("dataCode") String dataCode,@Param("startTime")  String startTime,@Param("endTime")  String endTime);

    List<ElectricityDataDto> selectCollectByMonth(@Param("preTime")String preTime,@Param("endTime") String endTime, @Param("enterpriseCode")String enterpriseCode);

    List<ElectricityDataVo> selectCollectByMonthEnt( @Param("enterpriseCode")String enterpriseCode,@Param("preTime")String preTime,@Param("endTime") String endTime);

    List<ElectricityDataVo> getMaterialElectricEnt(@Param("enterpriseCode") String enterpriseCode, @Param("startTime") String startTime, @Param("endTime")String endTime);

    List<Float> selectDataCode(@Param("enterpriseCode")String enterpriseCode,@Param("time") String time);

    List<ElectricityDataVo> selectOutput(String enterpriseCode);

    List<CoalVo> selectCollectData( @Param("areaCode")String areaCode, @Param("industryCode")String industryCode, @Param("preTime")String substring, @Param("endTime") String substring1);

    List<CoalVo> selectCollectData2019(@Param("areaCode")String areaCode, @Param("industryCode")String industryCode, @Param("preTime")String substring, @Param("endTime") String substring1);

    KeyValueVO getEleDataCode(@Param("enterpriseCode") String enterpriseCode);
}
